@@ -6,7 +6,7 @@
 ;; URL: http://github.com/s-kostyaev/elisa
 ;; Keywords: help local tools
 ;; Package-Requires: ((emacs "29.2") (ellama "0.11.2") (llm "0.9.1") (async "1.9.8") (plz "0.9"))
-;; Version: 1.0.4
+;; Version: 1.1.0
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 ;; Created: 18th Feb 2024
 
@@ -197,6 +197,10 @@ How to buy a pony?
   "Prompt template for prompt rewriting."
   :type 'string)
 
+(defcustom elisa-tika-url "http://localhost:9998/"
+  "Apache tika url for file parsing."
+  :type 'string)
+
 (defcustom elisa-searxng-url "http://localhost:8080/"
   "Searxng url for web search.  Json format should be enabled for this instance."
   :type 'string)
@@ -209,6 +213,10 @@ How to buy a pony?
   "Function to get buffer with webpage content."
   :type 'function)
 
+(defcustom elisa-complex-file-extraction-function #'elisa-parse-with-tika-buffer
+  "Function to get buffer with complex file (like pdf, odt etc.) content."
+  :type 'function)
+
 (defcustom elisa-web-search-function #'elisa-search-duckduckgo
   "Function to search the web.
 Function should get prompt and return list of urls."
@@ -260,6 +268,15 @@ If set, all quotes with similarity less than threshold will be filtered out."
   "Enabled collections for elisa chat."
   :type '(repeat string))
 
+(defcustom elisa-supported-complex-document-extensions '("doc" "dot" "ppt" "xls" "rtf" "docx" "pptx" "xlsx" "xlsm" "pdf" "epub" "msg" "odt" "odp" "ods" "odg" "docm")
+  "Supported complex document file extensions."
+  :type '(repeat string))
+
+(defun elisa-supported-complex-document-p (path)
+  "Check if PATH contain supported complex document."
+  (cl-find (file-name-extension path)
+	   elisa-supported-complex-document-extensions :test #'string=))
+
 (defun elisa-sqlite-vss-download-url ()
   "Generate sqlite vss download url based on current system.
 Sqlite vss is an extension to sqlite providing vector search
@@ -712,14 +729,18 @@ than T, it will be packed into single semantic chunk."
 		  (and (not (seq-some (lambda (regexp)
 					(string-match-p regexp file))
 				      ignore-regexps))
-		       (elisa--text-file-p file)))
+		       (or
+			(elisa-supported-complex-document-p file)
+			(elisa--text-file-p file))))
 		(directory-files-recursively directory ".*"))))
 
 (defun elisa-parse-file (collection-id path &optional force)
   "Parse file PATH for COLLECTION-ID.
 When FORCE parse even if already parsed."
   (let* ((opened (get-file-buffer path))
-	 (buf (or opened (find-file-noselect path t t)))
+	 (buf (if (elisa-supported-complex-document-p path)
+		  (funcall elisa-complex-file-extraction-function path)
+		(or opened (find-file-noselect path t t))))
 	 (hash (secure-hash 'sha256 buf))
 	 (prev-hash (caar (sqlite-select
 			   elisa-db
@@ -870,6 +891,63 @@ When FORCE parse even if already parsed."
 	  'a))
 	:test #'string-equal)))))
 
+(defun elisa-starts-with-lowercase-p (string)
+  "Check if STRING start with lowercase character."
+  (let ((category (get-char-code-property (seq-first string) 'general-category)))
+    (or (eq 'Ll category)
+	(eq 'Ps category))))
+
+(defun elisa-dehyphen (text)
+  "Dehyphen TEXT."
+  (ignore-errors (with-temp-buffer
+		   (insert (string-join
+			    (mapcar #'string-trim (string-split text "\n"))
+			    "\n"))
+		   (goto-char (point-min))
+		   (while (not (eobp))
+		     (end-of-line)
+		     (if (eq (preceding-char) ?-)
+			 (progn
+			   (delete-char 1)
+			   (delete-char -1))
+		       (forward-line)))
+		   (buffer-substring-no-properties (point-min) (point-max)))))
+
+(defun elisa-parse-with-tika-buffer (file)
+  "Parse FILE with tika."
+  (let* ((url (format "%s/tika" (string-trim-right elisa-tika-url "/")))
+	 (buf (plz 'put url :body (list 'file file) :as 'buffer))
+	 (shr-use-fonts nil)
+	 (shr-width (- ellama-long-lines-length 5))
+	 (data (with-current-buffer buf
+		 (libxml-parse-html-region (point-min) (point-max))))
+	 (prev-elt nil))
+    (dolist (elt (dom-by-tag data 'p))
+      (dolist (text (dom-children elt))
+	;; trim string content
+	(when-let* ((trimmed-text (string-trim text))
+		    (new-elt (if (or (string-match "^[0-9]+$" trimmed-text)
+				     (string= "" trimmed-text))
+				 (progn (dom-remove-node data elt)
+					nil)
+			       (if (elisa-starts-with-lowercase-p trimmed-text)
+				   (progn
+				     (dom-remove-node data prev-elt)
+				     (dom-node 'p nil (elisa-dehyphen
+						       (concat
+							(car (dom-children prev-elt))
+							"\n" trimmed-text))))
+				 (dom-node 'p nil (elisa-dehyphen trimmed-text))))))
+	  (setq prev-elt new-elt)
+	  (setq data (cl-nsubst new-elt elt data :test #'equal))))
+      (when (eq (length (dom-children elt)) 0)
+	(dom-remove-node data elt)))
+    (with-current-buffer buf
+      (delete-region (point-min) (point-max))
+      (ignore-errors
+	(shr-insert-document data))
+      buf)))
+
 (defun elisa-search-searxng (prompt)
   "Search searxng for PROMPT and return list of urls.
 You can customize `elisa-searxng-url' to use non local instance."
@@ -1166,7 +1244,10 @@ Call ON-DONE callback with result as an argument after FUNC evaluation done."
 		    ,(async-inject-variables "elisa-rewrite-prompt-template")
 		    ,(async-inject-variables "elisa-semantic-split-function")
 		    ,(async-inject-variables "elisa-webpage-extraction-function")
+		    ,(async-inject-variables "elisa-supported-complex-document-extensions")
+		    ,(async-inject-variables "elisa-complex-file-extraction-function")
 		    ,(async-inject-variables "elisa-web-search-function")
+		    ,(async-inject-variables "elisa-tika-url")
 		    ,(async-inject-variables "elisa-searxng-url")
 		    ,(async-inject-variables "elisa-web-pages-limit")
 		    ,(async-inject-variables "elisa-breakpoint-threshold-amount")
